# -*- mode: asm; asm-comment-char: ?\# -*-
# \file mixer.S
# 
# Sound mixer of SOSS (Simple OSS Sound)
# 
# \author Pontus Lidman <pontus@lysator.liu.se>
# 
# This file is subject to the terms and conditions of the GNU General
# Public License. See the file "COPYING" in this directory for
# details.
# 
# 

#include "soss.h"

# conveniently available on mips-linux

#include <asm/regdef.h>
        
#        
# Useful macros saves some typing and is a nice way to
# use 'named registers'
#
	.macro getsample shiftreg, step, pos, posfr, sbase, nsamp, \
			 shiftreg2, vol, ramptmp, vramp, \
			 shiftreg3, volr, ramptmpr, vrampr, \
			 tmp, tmp2, tmp3, tmp4
        dsll    \shiftreg, \shiftreg, 16
        addu    \tmp, \pos, \sbase
        add     \posfr, \step           # add step to posfr
        lhu     \tmp2, 0(\tmp)
        sra     \tmp, \posfr, 8
        andi    \posfr, \posfr, 0xff    # new posfr
        add     \tmp, \tmp, \tmp
        or      \shiftreg, \shiftreg, \tmp2
        add     \pos, \tmp              # new pos (halfword)
        dsll    \shiftreg2, \shiftreg2, 16
        dsll    \shiftreg3, \shiftreg3, 16        
        sub     \tmp2, \pos, \nsamp
        sra     \tmp3, \ramptmp, 10
        sra     \tmp4, \ramptmpr, 10
        slt     \tmp, \tmp2, zero
        add     \tmp3, \tmp3, \vol
        add     \tmp4, \tmp4, \volr
        movz    \pos, \tmp2, \tmp
        andi    \tmp3, \tmp3, 0xffff
        andi    \tmp4, \tmp4, 0xffff
        add     \ramptmp, \ramptmp, \vramp
        or      \shiftreg3, \shiftreg3, \tmp4
        add     \ramptmpr, \ramptmpr, \vrampr
        or      \shiftreg2, \shiftreg2, \tmp3
        .endm                

        .macro vol_mult resl, resr, samples, voll, volr, tmp1, tmp2, tmp3
       
        pmulth  zero, \samples, \voll    # left channel volume multiply
        pnor    \tmp3, zero, zero
        pmfhl.uw \resl          # v0 = p7,p5,p3,p1
        pmfhl.lw \tmp1          # tmp1 = p6,p4,p2,p0
        pmulth  zero, \samples, \volr # right channel volume multiply
        pextlh  \tmp3, \tmp3, zero    # high halfword mask (f0f0f0f0)
        pmfhl.uw \resr          # \resr = p7,p5,p3,p1
        pmfhl.lw \tmp2          # \tmp2 = p6,p4,p2,p0
        mtsah   zero, 1         # 1 halfword right
        pand    \resl, \resl, \tmp3
        pand    \resr, \resr, \tmp3
        pand    \tmp1, \tmp1, \tmp3
        pand    \tmp2, \tmp2, \tmp3
        qfsrv   \tmp1, zero, \tmp1
        qfsrv   \tmp2, zero, \tmp2
        por     \resl, \resl, \tmp1
        por     \resr, \resr, \tmp2  # resl,resr = 8 volume-corrected samples, left/right
        .endm
                
#
# Mixer
#
                 
	.text
	.globl	mixer
	.p2align 3
        .type   mixer, @function
       	.ent	mixer, 0
mixer:  .frame sp, 64, ra

        subu   sp, sp, 64
        ## callee-saved registers
        sw      s0, 0(sp)
        sw      s1, 4(sp)
        sw      s2, 8(sp)
        sw      s3, 12(sp)
        sw      s4, 16(sp)
        sw      s5, 20(sp)
        sw      s6, 24(sp)
        sw      s7, 28(sp)
        sw      t9, 32(sp)        
        la      a0, channels

        ## Clear buffer before adding samples.
        
        lw      a1, backBuffer
        addiu   t9, zero, BUF_SAMPLES/4
.clear:         
        sq      zero, 0(a1)
        nop
        nop
        nop
        addiu   a1, a1, 16
        addi    t9, t9, -1
        bnez    t9, .clear

	addiu	t8, zero, NUM_CHAN
        .p2align 3              # at least get known instruction pairing state

.chanloop:
     	lw	a3, ch_samples(a0)
        beqz    a3, .noinstr    # no samples for this channel

        lw      a1, backBuffer
	addiu	t9, zero, BUF_SAMPLES/8

        ## sample get reg setup
        lw      s1, ch_nsamples(a0)
	lw	t5, ch_pos(a0)
        add     s1, s1, s1      # halfwords
	lw	t6, ch_posfr(a0)
	lw	t7, ch_step(a0)
	lw	t5, ch_pos(a0)
        
        ## volume ramp reg setup
	lw	s2, ch_volume_left(a0)
        move    s4, zero
	lw	s3, ch_vramp_l(a0)
        lw      s5, ch_volume_right(a0)
        move    s7, zero
        lw      s6, ch_vramp_r(a0)
.smploop:
        ## get samples into t0, left volume ramp in t1, 
        ## right volume ramp in s0 
        .set    noat
	getsample t0, t7, t5, t6, a3, s1, t1, s2, s4, s3, s0, s5, s7, s6, t3, t2, t4, AT
	getsample t0, t7, t5, t6, a3, s1, t1, s2, s4, s3, s0, s5, s7, s6, t3, t2, t4, AT
	getsample t0, t7, t5, t6, a3, s1, t1, s2, s4, s3, s0, s5, s7, s6, t3, t2, t4, AT
	getsample t0, t7, t5, t6, a3, s1, t1, s2, s4, s3, s0, s5, s7, s6, t3, t2, t4, AT
        pcpyld  t0, t0, zero
        pcpyld  t1, t1, zero
        pcpyld  s0, s0, zero
	getsample t0, t7, t5, t6, a3, s1, t1, s2, s4, s3, s0, s5, s7, s6, t3, t2, t4, AT
	getsample t0, t7, t5, t6, a3, s1, t1, s2, s4, s3, s0, s5, s7, s6, t3, t2, t4, AT
	getsample t0, t7, t5, t6, a3, s1, t1, s2, s4, s3, s0, s5, s7, s6, t3, t2, t4, AT
	getsample t0, t7, t5, t6, a3, s1, t1, s2, s4, s3, s0, s5, s7, s6, t3, t2, t4, AT
        .set at

        ## process the samples in parallel
        ## t0 = 8 samples
        ## t1 = 8 volume values for left channel
        ## s0 = 8 volume values for right channel
               
        vol_mult v0, v1, t0, t1, s0, t2, t3, t4

        ## get old samples
        lq      t2, 16(a1)
        mtsah   zero, 4
        lq      t3, 0(a1)

        ## reverse samples
        prevh   v0, v0
        prevh   v1, v1
        qfsrv   v0, v0, v0
        qfsrv   v1, v1, v1

        ## interleave left/right channels
        pextuh  t0, v1, v0
        pextlh  t1, v1, v0
        paddsh  t0, t0, t2
        paddsh  t1, t1, t3

        ## store result
        sq      t0, 16(a1)
        addi    t9, t9, -1
        sq      t1, 0(a1)
        
        addiu   a1, a1, 32
	bnez	t9, .smploop	

        sw	t5, ch_pos(a0)
        sw      t6, ch_posfr(a0)
        
.noinstr:
        ## next channel
        addi    t8, t8, -1
        addiu   a0, a0, ch_sizeof
       	bnez	t8, .chanloop

.done:
        
        lw      s0, 0(sp)
        lw      s1, 4(sp)
        lw      s2, 8(sp)
        lw      s3, 12(sp)
        lw      s4, 16(sp)
        lw      s5, 20(sp)
        lw      s6, 24(sp)
        lw      s7, 28(sp)
        lw      t9, 32(sp)
        addiu   sp, sp, 64
        jr      ra
        .end    mixer
        .size   mixer,.-mixer









