/*
 * \file music.c
 *
 * Simple music driver.
 *
 * This file is part of SOSS (Simple OSS Sound)
 *
 * \author Pontus Lidman <pontus@lysator.liu.se>
 *
 * This file is subject to the terms and conditions of the GNU General
 * Public License. See the file "COPYING" in this directory for
 * details.
 *
 */

#include <stdint.h>
#include <stdio.h>

#include "soss.h"

float freq[8*12] = {
    /* C-0 */
    16.35 , 17.32 , 18.35 , 19.45 , 20.60 , 21.83 ,
    23.12 , 24.50 , 25.96 , 27.50 , 29.14 , 30.87 ,

    /* C-1 */
    32.70 , 34.65 , 36.71 , 38.89 , 41.20 , 43.65 ,
    46.25 , 49.00 , 51.91 , 55.00 , 58.27 , 61.74 ,

    /* C-2 */
    65.41 , 69.30 , 73.42 , 77.78 , 82.41 , 87.31 ,
    92.50 , 98.00 , 103.83, 110.00, 116.54, 123.47,

    /* C-3 */
    130.81, 138.59, 146.83, 155.56, 164.81, 174.61,
    185.00, 196.00, 207.65, 220.00, 233.08, 246.94,

    /* C-4 */
    261.63, 277.18, 293.66, 311.13, 329.63, 349.23,
    369.99, 392.00, 415.30, 440.00, 466.16, 493.88,

    /* C-5 */
    523.25, 554.37, 587.33, 622.25, 659.26, 698.46,
    739.99, 783.99, 830.61, 880.00, 932.33, 987.77,

    /* C-6 */
    1046.50,1108.73,1174.66,1244.51,1318.51,1396.91,
    1479.98,1567.98,1661.22,1760.00,1864.66,1975.53,

    /* C-7 */
    2093.00,2217.46,2349.32,2489.02,2637.02,2793.83,
    2959.96,3135.96,3322.44,3520.00,3729.31,3951.07
};

uint32_t ifreq[8*12];


typedef struct { int period; int note; int keydown; } note_t;

#include "moonlight.c"

#define MUSIC_CHAN 5
static note_t *tracks[MUSIC_CHAN] = { chan0, chan1, chan2, chan3, chan4 };

static int pointer[MUSIC_CHAN];
static int delay[MUSIC_CHAN];
static int channelnr[MUSIC_CHAN];

void music_init(void)
{
    int i;
    for (i=0; i<8*12; i++) {
	ifreq[i] = (freq[i]*SAMPLE_RATE)/64;
    }
    for (i=0; i<MUSIC_CHAN; i++) {
	channelnr[i] = 2*i;
    }
}

void music(void)
{
    note_t n;
    uint32_t f;
    int i;
    int chn;

    //channels[2].ext->master = 65536/6;
    for (i=0; i<MUSIC_CHAN; i++) {
	if (delay[i] == 0) {
	    n = tracks[i][pointer[i]];
	    if (n.keydown && i != 1)
		channelnr[i] ^= 1;
	    if (n.note >= 0) {
		f = ifreq[n.note];
		chn = channelnr[i];
		switch(i) {
		case 0:
		case 1:
		    play_note(&triinstr, f, chn, n.keydown);
		    break;
		default:
		    play_note(&sininstr, f, chn, n.keydown);
		}
	    }
	    delay[i] = n.period;
	    pointer[i]++;
	    if (n.note == -2)
		pointer[i] = 0;
	}
	delay[i]--;
    }
}
