/*
 * \file soss.h
 *
 * Header file for Simple OSS Sound. This file is included both by the
 * C files and the assembly language files. In the later case,
 * __ASSEMBLY__ should be defined on the compiler command line.
 *
 * \author Pontus Lidman <pontus@lysator.liu.se>
 *
 * This file is subject to the terms and conditions of the GNU General
 * Public License. See the file "COPYING" in this directory for
 * details.
 *
 */

/* Native sample rate of PS2 SPU under Linux/OSS */
#define SAMPLE_RATE 48000

/* Designed to run at 60Hz */
#define BUF_SAMPLES (SAMPLE_RATE/60)

/* Can probably be changed */
#define NUM_CHAN 10

#ifndef __ASSEMBLY__

/* Stereo samples */
typedef struct {
    int16_t left;
    int16_t right;
} stereo_t;

/* Mono samples */
typedef int16_t mono_t;

#endif

#ifndef __ASSEMBLY__

/* An instrument with waveform and envelope */
typedef struct {
    mono_t *samples;
    uint32_t nsamples;
    uint32_t attack;
    uint32_t decay;
    uint32_t sustain;
    uint32_t release;
    int32_t pan_start;
    int32_t pan_end;
    int32_t pan_duration;
} instrument_t;

/* Channel stuff not needed in the asm code */
typedef struct {
    instrument_t *instr;
    uint32_t envpos;
    uint32_t keydown;
    int32_t releaselevel;
    uint32_t releasetime;
    int32_t adsrvolume;
    int32_t master;
} chanext_t;

/* Channel stuff */
typedef struct {
    mono_t *samples;
    uint32_t nsamples;
    uint32_t step;
    uint32_t pos;
    uint32_t posfr;
    int32_t volume_left;
    int32_t volume_right;
    int32_t vramp_l;
    int32_t ramptmp_l;
    int32_t vramp_r;
    int32_t ramptmp_r;
    chanext_t *ext;
} channel_t;


#else
		.struct 0
instr_samples:	.struct instr_samples+4
instr_nsamples:	.struct instr_nsamples+4
instr_sizeof:

		.struct 0
ch_samples:	.struct ch_samples+4
ch_nsamples:	.struct ch_nsamples+4
ch_step:	.struct ch_step+4
ch_pos:		.struct ch_pos+4
ch_posfr:	.struct ch_posfr+4
ch_volume_left:	.struct ch_volume_left+4
ch_volume_right: .struct ch_volume_right+4
ch_vramp_l:	.struct ch_vramp_l+4
ch_ramptmp_l:	.struct ch_ramptmp_l+4
ch_vramp_r:	.struct ch_vramp_r+4
ch_ramptmp_r:	.struct ch_ramptmp_r+4
ch_ext:		.struct ch_ext+4
ch_sizeof:
#endif

#ifndef __ASSEMBLY__

extern instrument_t sininstr;
extern instrument_t sin2instr;
extern instrument_t sin3instr;
extern instrument_t sqinstr;
extern instrument_t sawinstr;
extern instrument_t noiseinstr;
extern instrument_t triinstr;

extern channel_t channels[NUM_CHAN];

void music(void);
void music_init(void);

void keyDown(instrument_t *instr, uint32_t freq, int chan);
void keyUp(int chan);
void mixer(void);
void c_mixer(void);
void initsamples();
void channel_init();
void envelope();
void soundoutput();
int oss_open();
void play_note(instrument_t *instr, uint32_t freq, int chan, int keydown);
#endif
