/*
 * \file speedtest.c
 *
 * Measure the speed of the mixer routine.
 *
 * This file is part of SOSS (Simple OSS Sound)
 *
 * \author Pontus Lidman <pontus@lysator.liu.se>
 *
 * This file is subject to the terms and conditions of the GNU General
 * Public License. See the file "COPYING" in this directory for
 * details.
 *
 */

#include <stdio.h>
#include <stdint.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/time.h>

#include "soss.h"

#define LOOPS 10000
#define NUM_INSTR 5
instrument_t *instrs[NUM_INSTR]= { 
    &sininstr, &triinstr, &sawinstr, &noiseinstr, &sqinstr 
};

int main(int argc, char **argv)
{
    int i, j;
    struct timeval before, after;
    uint64_t t1, t2;
    uint64_t per_sec;

    initsamples();
    channel_init();

    /* put a little bit of something in each channel */
    for (i=0, j=0; i<NUM_CHAN; i++, j = (++j % NUM_INSTR)) {
        play_note(instrs[j], 22050+i*(96000/NUM_CHAN), i, 1);
    }
    envelope();

    gettimeofday(&before, 0);
    for (i=0; i<LOOPS; i++) {
#ifdef USE_ASM
	mixer();
#else
	c_mixer();
#endif
    }
    gettimeofday(&after, 0);
    t1 = before.tv_sec * 1000000 + before.tv_usec;
    t2 = after.tv_sec * 1000000 + after.tv_usec;
    printf("%d loops in %lld microseconds.\n", LOOPS, t2-t1);
    per_sec = (LOOPS * 1000000ULL) / (t2-t1);
    printf("%lld loops per second.\n", per_sec);
    return 0;
}
